"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const npm_resolver_1 = require("@pnpm/npm-resolver");
const plugin_commands_publishing_1 = require("@pnpm/plugin-commands-publishing");
const run_npm_1 = require("@pnpm/run-npm");
const store_path_1 = require("@pnpm/store-path");
const utils_1 = require("@pnpm/utils");
const LRU = require("lru-cache");
const pFilter = require("p-filter");
async function default_1(pkgs, opts) {
    var _a;
    const storeDir = await store_path_1.default(opts.workspaceDir, opts.storeDir);
    const resolve = npm_resolver_1.default(Object.assign(opts, {
        fullMetadata: true,
        metaCache: new LRU({
            max: 10000,
            maxAge: 120 * 1000,
        }),
        storeDir,
    }));
    const pkgsToPublish = await pFilter(pkgs, async (pkg) => {
        if (!pkg.manifest.name || !pkg.manifest.version)
            return false;
        return !(await isAlreadyPublished({
            dir: pkg.dir,
            lockfileDir: opts.lockfileDir || pkg.dir,
            registries: opts.registries,
            resolve,
        }, pkg.manifest.name, pkg.manifest.version));
    });
    for (const pkg of pkgsToPublish) {
        const access = (_a = opts.access, (_a !== null && _a !== void 0 ? _a : (pkg.manifest.name.startsWith('@') ? 'restricted' : 'public')));
        await plugin_commands_publishing_1.publish.handler([pkg.dir], {
            argv: {
                original: [
                    'publish',
                    pkg.dir,
                    '--tag',
                    'pnpm-temp',
                    '--registry',
                    utils_1.pickRegistryForPackage(opts.registries, pkg.manifest.name),
                    '--access',
                    access,
                ],
            },
            workspaceDir: opts.workspaceDir,
        }, 'publish');
    }
    const tag = opts.tag || 'latest';
    for (const pkg of pkgsToPublish) {
        await run_npm_1.default([
            'dist-tag',
            'add',
            `${pkg.manifest.name}@${pkg.manifest.version}`,
            tag,
            '--registry',
            utils_1.pickRegistryForPackage(opts.registries, pkg.manifest.name),
        ]);
    }
}
exports.default = default_1;
async function isAlreadyPublished(opts, pkgName, pkgVersion) {
    try {
        await opts.resolve({ alias: pkgName, pref: pkgVersion }, {
            importerDir: opts.dir,
            lockfileDir: opts.lockfileDir,
            preferredVersions: {},
            registry: utils_1.pickRegistryForPackage(opts.registries, pkgName),
        });
        return true;
    }
    catch (err) {
        return false;
    }
}
